﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class Tile : MonoBehaviour {

    private BoxCollider2D c2d;

    public Vector2 place;

    public bool canUp;
    public bool canDown;
    public bool canLeft;
    public bool canRight;
    public Dictionary<Vector2, bool> ableDirection = new Dictionary<Vector2, bool>();

    public Tile[] adjacentTiles = new Tile[4];
    public Dictionary<Vector2, bool> ableMovement = new Dictionary<Vector2, bool>();

    protected virtual void Awake() {
        c2d = GetComponent<BoxCollider2D>();

        ableMovement.Add(Vector2.up, false);
        ableMovement.Add(Vector2.down, false);
        ableMovement.Add(Vector2.left, false);
        ableMovement.Add(Vector2.right, false);

        ableDirection.Add(Vector2.up, canUp);
        ableDirection.Add(Vector2.down, canDown);
        ableDirection.Add(Vector2.left, canLeft);
        ableDirection.Add(Vector2.right, canRight);
    }

    // Use this for initialization
    protected void Start () {
        FindAdjacent();
    }

    // Update is called once per frame
    protected void Update () {
        
    }

    public virtual bool ManipulateToken(Token token, Vector2 from) {
        //Debug.Log("Manipulate " + token.gameObject.name + " from" + from);
        
        if (ableMovement[token.defaultDirection]) {
            
            if (TileManager.Instance.GetTileAt(place - token.defaultDirection) == null) {
                switch (token.gameObject.name) {
                    case "Evil":
                        TileManager.Instance.evilGoal.transform.position
                            = new Vector3(token.transform.position.x, -TileManager.Instance.GetBorderY());
                        token.targetTile = TileManager.Instance.evilGoal;
                        break;
                    case "Rocket":
                        TileManager.Instance.rocketGoal.transform.position
                            = new Vector3(token.transform.position.x, TileManager.Instance.GetBorderY());
                        token.targetTile = TileManager.Instance.rocketGoal;
                        break;
                }
                token.ReachEnd();
                return true;
            }
            
            token.direction = token.defaultDirection;
            token.targetTile = TileManager.Instance.GetTileAt(place - token.defaultDirection);
            return true;
        }

        if (from == Vector2.left) {
            if (ableMovement[Vector2.right]) {
                token.direction = Vector2.right;
                token.targetTile = adjacentTiles[3];
                return true;
            }
        }

        if (from == Vector2.right) {
            if (ableMovement[Vector2.left]) {
                token.direction = Vector2.left;
                token.targetTile = adjacentTiles[2];
                return true;
            }
        }

        if (from == -token.defaultDirection) {
            if (ableMovement[Vector2.left] && ableMovement[Vector2.right]) {
                token.direction = Vector2.zero;
                return false;
            } else if (ableMovement[Vector2.left]) {
                token.direction = Vector2.left;
                token.targetTile = adjacentTiles[2];
                return true;
            } else if (ableMovement[Vector2.right]) {
                token.direction = Vector2.right;
                token.targetTile = adjacentTiles[3];
                return true;
            }
        }

        return false;
    }

    public void FindAdjacent() {
        //Up
        adjacentTiles[0] = place.y == 1? null : TileManager.Instance.GetTileAt(new Vector2(place.x, place.y - 1));

        //Down
        adjacentTiles[1] = place.y == 6 ? null : TileManager.Instance.GetTileAt(new Vector2(place.x, place.y + 1));

        //Left
        adjacentTiles[2] = place.x == 1 ? TileManager.Instance.GetTileAt(new Vector2(3, place.y)) 
                                        : TileManager.Instance.GetTileAt(new Vector2(place.x - 1, place.y));

        //Right
        adjacentTiles[3] = place.x == 3 ? TileManager.Instance.GetTileAt(new Vector2(1, place.y)) 
                                        : TileManager.Instance.GetTileAt(new Vector2(place.x + 1, place.y));

        UpdateAbleMovement();
    }

    public void UpdateAbleMovement() {
        if ((adjacentTiles[0] == null && ableDirection[Vector2.up])
                    || (ableDirection[Vector2.up] && adjacentTiles[0].ableDirection[Vector2.down])) {
            ableMovement[Vector2.up] = true;
            if (adjacentTiles[0] != null)
                adjacentTiles[0].ableMovement[Vector2.down] = true;
        } else {
            ableMovement[Vector2.up] = false;
            if (adjacentTiles[0] != null)
                adjacentTiles[0].ableMovement[Vector2.down] = false;
        }

        if ((adjacentTiles[1] == null && ableDirection[Vector2.down])
                    || (ableDirection[Vector2.down] && adjacentTiles[1].ableDirection[Vector2.up])) {
            ableMovement[Vector2.down] = true;
            if (adjacentTiles[1] != null)
                adjacentTiles[1].ableMovement[Vector2.up] = true;
        } else {
            ableMovement[Vector2.down] = false;
            if (adjacentTiles[1] != null)
                adjacentTiles[1].ableMovement[Vector2.up] = false;
        }

        if (ableDirection[Vector2.left] && adjacentTiles[2].ableDirection[Vector2.right]) {
            ableMovement[Vector2.left] = true;
        } else {
            ableMovement[Vector2.left] = false;
        }

        if (ableDirection[Vector2.right] && adjacentTiles[3].ableDirection[Vector2.left]) {
            ableMovement[Vector2.right] = true;
        } else {
            ableMovement[Vector2.right] = false;
        }
    }
    /*
    public void OnDrawGizmos() {
        if (ableMovement[Vector2.up]) {
            Gizmos.color = Color.green;
        } else {
            Gizmos.color = Color.red;
        }
        Gizmos.DrawCube((transform.position + new Vector3(0, 0.5f, 0)), 
                            new Vector3(transform.localScale.x, 0.2f, 1));

        if (ableMovement[Vector2.down]) {
            Gizmos.color = Color.green;
        } else {
            Gizmos.color = Color.red;
        }
        Gizmos.DrawCube((transform.position + new Vector3(0, -0.5f, 0)), 
                            new Vector3(transform.localScale.x, 0.2f, 1));

        if (ableMovement[Vector2.left])
        {
            Gizmos.color = Color.green;
        }
        else
        {
            Gizmos.color = Color.red;
        }
        Gizmos.DrawCube((transform.position + new Vector3(-2.5f, 0, 0)),
                            new Vector3(0.2f, transform.localScale.y, 1));

        if (ableMovement[Vector2.right])
        {
            Gizmos.color = Color.green;
        }
        else
        {
            Gizmos.color = Color.red;
        }
        Gizmos.DrawCube((transform.position + new Vector3(2.5f, 0, 0)),
                            new Vector3(0.2f, transform.localScale.y, 1));
    }
    */
}
