﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class Token : MonoBehaviour {

    public float speed;
    public Vector2 direction;
    public Vector2 enterDirection;

    public Vector3 defaultPosition;
    public Vector2 defaultDirection;
    public Tile defaultTargetTile;
    public Vector2 defaultPlace;

    public Vector2 stayPlace;
    public Tile targetTile;

    public bool canMove = false;
    public bool isMove = false;
    public bool isEnd = false;

    public GameObject[] redLines;
    public GameObject[] greenLines;

    [SerializeField]
    private float t;
    private Vector3 lastPosition;

    protected void Awake() {
        defaultPosition = transform.position;
        lastPosition = transform.position;
        foreach (GameObject line in redLines) {
            line.SetActive(false);
        }
        foreach (GameObject line in greenLines) {
            line.SetActive(false);
        }
    }

    // Use this for initialization
    protected void Start () {
        direction = defaultDirection;
    }
	
	// Update is called once per frame
	protected void FixedUpdate () {
        if (!isMove) {
            transform.position = new Vector3(transform.position.x, transform.position.y, 
                                                transform.position.z == 0 ? 1 : 0);
        }
	}

    protected void Move() {
        //transform.position = new Vector3(transform.position.x + direction.x * speed,
        //                                    transform.position.y + direction.y * speed, 0);
        
        transform.position = Vector3.Lerp(lastPosition, targetTile.transform.position, t);
        t += speed;
        if (t >= 1) {
            isMove = false;
        }
    }

    public IEnumerator MoveTo(Vector3 target, float speed) {
        isMove = true;

        lastPosition = transform.position;
        t = 0;
        while (t < 1) {
            transform.position = Vector3.Lerp(lastPosition, target, t);
            t += speed;
            yield return null;
        }

        isMove = false;
    }

    public IEnumerator Across(int sign) {
        isMove = true;

        lastPosition = transform.position;
        t = 0;
        while (t < 1)
        {
            transform.position = Vector3.Lerp(lastPosition, new Vector3(sign * TileManager.Instance.GetBorderX(), lastPosition.y), t);
            t += speed * 2;
            yield return null;
        }

        transform.position = new Vector3(-sign * TileManager.Instance.GetBorderX(), lastPosition.y);
        lastPosition = transform.position;
        t = 0;
        while (t < 1)
        {
            transform.position = Vector3.Lerp(lastPosition, targetTile.transform.position, t);
            t += speed * 2;
            yield return null;
        }

        isMove = false;
    }

    protected virtual void OnTriggerStay2D(Collider2D collision) {
        //Debug.Log(collision.gameObject.name + " " + gameObject.name + " "+ isMove);

        if (collision.gameObject.tag == "Tile") {
            stayPlace = collision.gameObject.GetComponent<Tile>().place;
            //Debug.Log("stayyyy " + collision.gameObject.name + " " + gameObject.name);
        }

        if (collision.gameObject.tag == "Tile" && !isMove && canMove) {
            Debug.Log("stay " + collision.gameObject.name + " " + gameObject.name);
            if (collision.gameObject.GetComponent<Tile>().ManipulateToken(this, -direction)) {

                if (targetTile.place.x == 3 && stayPlace.x == 1) {
                    StartCoroutine(Across(-1));
                }
                else if (targetTile.place.x == 1 && stayPlace.x == 3) {
                    StartCoroutine(Across(1));
                }
                else {
                    StartCoroutine(MoveTo(targetTile.transform.position, speed));
                }

            } else {
                canMove = false;
                OnLost();
            }
        }
        /*
        if (collision.gameObject.tag == "Target" && transform.position == collision.transform.position) {
            if (collision.gameObject.name == gameObject.name) {
                OnSuccess();
            } else {
                OnStuck();
            }
        }
        */
        //Debug.Log(collision.gameObject.name);

        
    }

    protected virtual void OnTriggerEnter2D(Collider2D collision) {
        if (collision.gameObject.tag == "Tile") {
            stayPlace = collision.gameObject.GetComponent<Tile>().place;
        }
    }

    protected virtual void OnTriggerExit2D(Collider2D collision) {
        if (collision.gameObject.tag == "Tile") {
            stayPlace = defaultPlace;
        }
    }

    public virtual void OnBeginMove() {
        if (targetTile.ableMovement[-defaultDirection]) {
            StartCoroutine(MoveTo(targetTile.transform.position, speed));
        } else {
            OnLost();
        }
    }

    public virtual void OnLost() {}

    public virtual void SetState(int num) {
        GetComponent<Animator>().SetInteger("state", num);
        SetRenderOrder(20);
    }

    public virtual void ReachEnd() {
        isEnd = true;

    }

    public void SetRenderOrder(int order) {
        GetComponent<SpriteRenderer>().sortingOrder = order;
    }

    public virtual void Reset() {
        StopAllCoroutines();
        transform.position = defaultPosition;
        canMove = false;
        targetTile = defaultTargetTile;
        t = 0;
        lastPosition = defaultPosition;
        direction = defaultDirection;
        SetState(0);
        isEnd = false;
        GetComponent<SpriteRenderer>().sortingOrder = 10;

        foreach (GameObject line in redLines) {
            line.SetActive(false);
        }
        foreach (GameObject line in greenLines) {
            line.SetActive(false);
        }
    }

}
